@@ -18,6 +18,7 @@ import cloudpickle
 import dis
 import functools
 import inspect
+import mmap
 import os
 import pathlib
 import pickle
@@ -227,7 +228,7 @@ if (
             result = subprocess.run(args)
             if result.returncode < 0:
                 print(
-                    "scalene error: received signal",
+                    "Scalene error: received signal",
                     signal.Signals(-result.returncode).name,
                 )
             sys.exit(result.returncode)
@@ -246,17 +247,20 @@ if (
             result = subprocess.run(args, close_fds=True, shell=False)
             if result.returncode < 0:
                 print(
-                    "scalene error: received signal",
+                    "Scalene error: received signal",
                     signal.Signals(-result.returncode).name,
                 )
             sys.exit(result.returncode)
 
 # Install our profile decorator.
 
-def scalene_redirect_profile(func):
+
+def scalene_redirect_profile(func: Any) -> Any:
     return Scalene.profile(func)
 
-builtins.profile = scalene_redirect_profile
+
+builtins.profile = scalene_redirect_profile  # type: ignore
+
 
 class Scalene:
     """The Scalene profiler itself."""
@@ -294,11 +298,14 @@ class Scalene:
     def get_original_lock() -> threading.Lock:
         return Scalene.__original_lock()
 
-    # Likely names for the Python interpreter (assuming it's the same version as this one).
+    # Likely names for the Python interpreter.
     __all_python_names = [
-        "python",
-        "python" + str(sys.version_info.major),
-        "python" + str(sys.version_info.major) + "." + str(sys.version_info.minor),
+        os.path.basename(sys.executable),
+        os.path.basename(sys.executable) + str(sys.version_info.major),
+        os.path.basename(sys.executable)
+        + str(sys.version_info.major)
+        + "."
+        + str(sys.version_info.minor),
     ]
 
     # Statistics counters:
@@ -421,12 +428,20 @@ class Scalene:
     __malloc_signal_position = 0
     try:
         __malloc_signal_fd = open(__malloc_signal_filename, "x")
-    except BaseException:
+    except BaseException as exc:
         pass
-    __malloc_signal_fd = open(__malloc_signal_filename, "r")
-    
+    try:
+        __malloc_signal_fd = open(__malloc_signal_filename, "r")
+        __malloc_signal_mmap = mmap.mmap(
+            __malloc_signal_fd.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ
+        )
+    except BaseException as exc:
+        # Ignore if we aren't profiling memory.
+        pass
+
     #   file to communicate the number of memcpy samples (+ PID)
     __memcpy_signal_filename = Filename("/tmp/scalene-memcpy-signal" + str(os.getpid()))
+    __memcpy_signal_fd = None
     try:
         __memcpy_signal_fd = open(__memcpy_signal_filename, "r")
         __memcpy_signal_position = 0
@@ -445,18 +460,6 @@ class Scalene:
     # Whether we are in a signal handler or not (to make things properly re-entrant).
     __in_signal_handler = threading.Lock()
 
-    # We cache the previous signal handlers so we can play nice with
-    # apps that might already have handlers for these signals.
-    __original_malloc_signal_handler: Union[
-        Callable[[Signals, FrameType], None], int, Handlers, None
-    ] = signal.SIG_IGN
-    __original_free_signal_handler: Union[
-        Callable[[Signals, FrameType], None], int, Handlers, None
-    ] = signal.SIG_IGN
-    __original_memcpy_signal_handler: Union[
-        Callable[[Signals, FrameType], None], int, Handlers, None
-    ] = signal.SIG_IGN
-
     # Program-specific information:
     #   the name of the program being profiled
     __program_being_profiled = Filename("")
@@ -551,16 +554,9 @@ start the timer interrupts."""
         # CPU
         signal.signal(Scalene.__cpu_signal, Scalene.cpu_signal_handler)
         # Set signal handlers for memory allocation and memcpy events.
-        # Save the previous signal handlers, if any.
-        Scalene.__original_malloc_signal_handler = signal.signal(
-            Scalene.__malloc_signal, Scalene.malloc_signal_handler
-        )
-        Scalene.__original_free_signal_handler = signal.signal(
-            Scalene.__free_signal, Scalene.free_signal_handler
-        )
-        Scalene.__original_memcpy_signal_handler = signal.signal(
-            Scalene.__memcpy_signal, Scalene.memcpy_event_signal_handler
-        )
+        signal.signal(Scalene.__malloc_signal, Scalene.malloc_signal_handler)
+        signal.signal(Scalene.__free_signal, Scalene.free_signal_handler)
+        signal.signal(Scalene.__memcpy_signal, Scalene.memcpy_event_signal_handler)
         # Set every signal to restart interrupted system calls.
         signal.siginterrupt(Scalene.__cpu_signal, False)
         signal.siginterrupt(Scalene.__malloc_signal, False)
@@ -943,10 +939,12 @@ start the timer interrupts."""
         # Process the input array from where we left off reading last time.
         arr: List[Tuple[int, str, float, float]] = []
         try:
-            mfile = Scalene.__malloc_signal_fd
-            mfile.seek(Scalene.__malloc_signal_position)
-            for count_str in mfile:
-                count_str = count_str.rstrip()
+            mm = Scalene.__malloc_signal_mmap
+            mm.seek(Scalene.__malloc_signal_position)
+            while True:
+                count_str = mm.readline().rstrip().decode("ascii")
+                if count_str == "":
+                    break
                 (
                     action,
                     alloc_time_str,
@@ -961,7 +959,7 @@ start the timer interrupts."""
                         float(python_fraction_str),
                     )
                 )
-            Scalene.__malloc_signal_position = mfile.tell()
+            Scalene.__malloc_signal_position = mm.tell()
         except FileNotFoundError:
             pass
 
@@ -1043,13 +1041,14 @@ start the timer interrupts."""
         arr: List[Tuple[int, int]] = []
         try:
             mfile = Scalene.__memcpy_signal_fd
-            mfile.seek(Scalene.__memcpy_signal_position)
-            for count_str in mfile:
-                count_str = count_str.rstrip()
-                (memcpy_time_str, count_str2) = count_str.split(",")
-                arr.append((int(memcpy_time_str), int(count_str2)))
-            Scalene.__memcpy_signal_position = mfile.tell()
-        except FileNotFoundError:
+            if mfile:
+                mfile.seek(Scalene.__memcpy_signal_position)
+                for count_str in mfile:
+                    count_str = count_str.rstrip()
+                    (memcpy_time_str, count_str2) = count_str.split(",")
+                    arr.append((int(memcpy_time_str), int(count_str2)))
+                Scalene.__memcpy_signal_position = mfile.tell()
+        except Exception as exc:
             pass
         arr.sort()
 
@@ -1489,10 +1488,14 @@ start the timer interrupts."""
     @staticmethod
     def disable_signals() -> None:
         """Turn off the profiling signals."""
-        signal.setitimer(Scalene.__cpu_timer_signal, 0)
-        signal.signal(Scalene.__malloc_signal, Scalene.__original_malloc_signal_handler)
-        signal.signal(Scalene.__free_signal, Scalene.__original_free_signal_handler)
-        signal.signal(Scalene.__memcpy_signal, Scalene.__original_memcpy_signal_handler)
+        try:
+            signal.setitimer(Scalene.__cpu_timer_signal, 0)
+            signal.signal(Scalene.__malloc_signal, signal.SIG_IGN)
+            signal.signal(Scalene.__free_signal, signal.SIG_IGN)
+            signal.signal(Scalene.__memcpy_signal, signal.SIG_IGN)
+        except:
+            # Retry just in case we get interrupted by one of our own signals.
+            Scalene.disable_signals()
 
     @staticmethod
     def exit_handler() -> None:
@@ -1536,6 +1539,8 @@ start the timer interrupts."""
                     the_globals = __main__.__dict__
                     # Splice in the name of the file being executed instead of the profiler.
                     the_globals["__file__"] = os.path.basename(sys.argv[0])
+                    # Some mysterious module foo to make this work the same with -m as with `scalene`.
+                    the_globals["__spec__"] = None
                     # Start the profiler.
                     fullname = os.path.join(program_path, os.path.basename(sys.argv[0]))
                     profiler = Scalene(Filename(fullname))
